package com.iot.modbus_rtcp;

import lombok.SneakyThrows;
import org.junit.jupiter.api.Test;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Objects;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/20 15:13
 */

public class SocketTest {

    private static final String HOST = "127.0.0.1";
    private static final Integer PORT = 1200;

    @Test
    @SneakyThrows
    public void testModbus() {
        try (Socket socket = new Socket(HOST, PORT, InetAddress.getByName(HOST), 10001);
             PrintWriter writer = new PrintWriter(socket.getOutputStream(), true);
             BufferedReader reader = new BufferedReader(new InputStreamReader(socket.getInputStream()))) {
            socket.setKeepAlive(true);
//            socket.setTcpNoDelay(true);
            socket.setSoTimeout(1500);
            String line = null;
            while (true) {
                if (socket.isClosed()) {
                    socket.close();
                    break;
                }

                try {
                    // 接收服务器响应
                    line = reader.readLine();
                    if (responseCommand(line, writer, socket)) break;
                } catch (Exception e) {
                }
            }
        }
    }

    private static boolean responseCommand(String line, PrintWriter writer, Socket socket) throws IOException {
        // 接收到请求
        if (Objects.equals(line, "01040000001531C5")) {
            // 发送响应字符串
            writer.println("01 04 2A 07 E8 00 0B 00 13 00 17 00 11 00 23 00 03 00 01 00 00 00 00 00 02 00 3B 03 E7 00 39 00 00 0D AC 0D AC 00 00 00 00 00 00 0A 14 4A 98".replaceAll(" ", ""));
        }
        // 接收到开井请求
        else if (Objects.equals(line, "01050001FF00DDFA")) {
            // 发送响应字符串
            writer.println("01050001FF00DDFA");
        }
        // 接收到关井请求
        else if (Objects.equals(line, "0105000100009C0A")) {
            // 发送响应字符串
            writer.println("0105000100009C0A");
        }
        // 接收到读取运行模式请求
        else if (Objects.equals(line, "010300640001C5D5")) {
            // 发送响应字符串
            writer.println("01 03 02 00 03 F8 45".replaceAll(" ", ""));
        } else {
            socket.close();
            return true;
        }
        return false;
    }
}
